#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-29
#
# monitor Check Point Eventia analyzer
#
# 2018-03-12: code cleanup
# 2020-06-07: split checkpoint_eva in to checkpoint_eva_cu_status and checkpoint_eva_cu_status
#             code cleanup
# 2020-06-08: changed snmp-scanfunction
# 2021-08-14: rewritten for CMK 2.0
#
# sample string_table SamrtEvent activated
# [[u'1', u'1', u'34221335', u'0', u'0', u'', u'']]
#
# SmartEvent running not local
# [[u'1', u'2', u'0', u'0', u'0', u'Not connected to SEM', u'Event distributor is not connected.']]
#
# sample string_table SmartEvent not activated
# [[u'0', u'0', u'0', u'0', u'2', u'CPSEAD not running', u'CPSEAD not running - process appears to be down']]
#

import time
from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    not_equals,
    get_rate,
    GetRateError,
    get_value_store,
    IgnoreResultsError,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointEvaCuStatus:
    procalive: int
    connectedtosem: int
    numprocessedlogs: int
    nofreediskspace: int
    statcode: int
    statshortdescr: str
    statlongdescr: str
    

def parse_checkpoint_eva_cu_status(string_table: StringTable) -> Optional[CheckpointEvaCuStatus]:
    try:
        procalive, connectedtosem, numprocessedlogs, nofreediskspace, statcode, statshortdescr, statlongdescr = \
        string_table[0]
    except (ValueError,IndexError):
        return

    return CheckpointEvaCuStatus(
        procalive=int(procalive),
        connectedtosem=int(connectedtosem),
        numprocessedlogs=int(numprocessedlogs),
        nofreediskspace=int(nofreediskspace),
        statcode=int(statcode),
        statshortdescr=statshortdescr,
        statlongdescr=statlongdescr,
    )


def discovery_checkpoint_eva_cu_status(section: CheckpointEvaCuStatus) -> DiscoveryResult:
    yield Service()


def check_checkpoint_eva_cu_status(section: CheckpointEvaCuStatus) -> CheckResult:
    now_time = time.time()
    value_store = get_value_store()

    try:
        numprocessedlogs_sec = get_rate(
            value_store,
            'checkpoint_eva_cu_status_numprocessedlogs',
            now_time,
            section.numprocessedlogs,
            raise_overflow=False
        )
    except GetRateError:
        raise IgnoreResultsError('Initializing counters')

    yield Result(state=State.OK, summary=f'# of processed logs: {section.numprocessedlogs}')
    yield from check_levels(
        label='Logs',
        value=numprocessedlogs_sec,
        metric_name='checkpoint_eva_numprocessedlogs_sec',
        render_func=lambda v: f'{v:.2f}/s'
    )

    if len(section.statlongdescr) != 0:
        yield Result(state=State.OK, summary=f'Status long: {section.statlongdescr}')

    if section.nofreediskspace != 0:
        yield Result(state=State.CRIT, notice='No free disk space')

    if section.procalive != 1:
        yield Result(state=State.CRIT, notice='Process not alive')

    if section.connectedtosem != 1:
        yield Result(state=State.CRIT, notice='Correlation unit not connected to server')

    if section.statcode != 0:
        yield Result(state=State.CRIT, notice=section.statshortdescr)


register.snmp_section(
    name='checkpoint_eva_cu_status',
    parse_function=parse_checkpoint_eva_cu_status,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.25.2',  # CHECKPOINT-MIB::cpsead
        oids=[
            '1',  # cpseadProcAlive
            '2',  # cpseadConnectedToSem
            '3',  # cpseadNumProcessedLogs
            '5',  # cpseadNoFreeDiskSpace
            '101',  # cpseadStatCode
            '102',  # cpseadStatShortDescr
            '103',  # cpseadStatLongDescr
        ]
    ),
    detect=all_of(
        any_of(
            startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
            all_of(
                equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
                equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
            )
        ),
        not_equals('.1.3.6.1.4.1.2620.1.25.2.101.0', '2')
    )
)

register.check_plugin(
    name='checkpoint_eva_cu_status',
    service_name='Correlation unit status',
    discovery_function=discovery_checkpoint_eva_cu_status,
    check_function=check_checkpoint_eva_cu_status,
)
