#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-30
#
# monitor Check Point Eventia analyzer database
#
# 2016-08-03: added warn/crit in percent/GB for free disk space and free db space
#             added max levels for perfdata
#             changed perfdata from bytes to MB
#             added pnp-template --> moved to metrics
# 2020-06-08: changed snmp-scan function
# 2021-08-12: rewritten for CMK 2.0
# 2022-09-08: fixed missing check_ruleset_name='checkpoint_eva_db',
#
#
# sample string_table
# [['632058124697', '0', '395742854', '226516575191', '0', '858574699888']]
#
# not activated
# [[u'0', u'0', u'0', u'0', u'0', u'0']]
#
# sample section
# CheckpointEvaDb(currentdbsize=632058124697, dbcapacity=0, dbdiskspace=226516575191,
# numevents=395742854, dbisfull=0, disksize=858574699888)
#

from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    equals,
    Result,
    State,
    check_levels,
    render,
    SNMPTree,
    startswith,
    all_of,
    any_of,
)


@dataclass
class CheckpointEvaDb:
    currentdbsize: int
    dbcapacity: int
    dbdiskspace: int
    numevents: int
    dbisfull: int
    disksize: int
    dbsize_percent: float


def parse_checkpoint_eva_db(string_table: StringTable) -> Optional[CheckpointEvaDb]:
    if string_table != [['0', '0', '0', '0', '0', '0']]:
        try:
            currentdbsize, dbcapacity, numevents, dbdiskspace, dbisfull, disksize = string_table[0]
        except (ValueError, IndexError):
            return

        if int(dbcapacity) == 0:  # missing data in version R80.10
            dbcapacity = int(dbdiskspace) + int(currentdbsize)

        return CheckpointEvaDb(
            currentdbsize=int(currentdbsize),
            dbcapacity=int(dbcapacity),
            dbdiskspace=int(dbdiskspace),
            numevents=int(numevents),
            dbisfull=int(dbisfull),
            disksize=int(disksize),
            dbsize_percent=100 / int(dbcapacity) * int(currentdbsize)
        )


def discovery_checkpoint_eva_db(section: CheckpointEvaDb) -> DiscoveryResult:
    yield Service()


def check_checkpoint_eva_db(params, section: CheckpointEvaDb) -> CheckResult:
    if section.dbisfull != 0:
        yield Result(state=State.CRIT, summary='DB is full')

    warn, crit = params['levels_db_size_percent']

    yield from check_levels(
        value=section.dbsize_percent,
        levels_upper=(warn, crit),
        label='DB disk space used',
        render_func=render.percent,
        metric_name='checkpoint_eva_db_dbsize_percent',
    )

    for metric, value, label in [
        ('db_size', section.currentdbsize, 'DB size'),  # None, None, 0, dbcapacity),
        ('db_disk_space', section.dbdiskspace, 'DB disk space free'),
        ('db_capacity', section.dbcapacity, 'DB capacity'),  # None, None, 0, section.dbdiskspace),
        ('disk_size', section.disksize, 'DB Disk size'),
    ]:
        yield from check_levels(
            label=label,
            value=value,
            metric_name=f'checkpoint_eva_db_{metric}',
            render_func=render.disksize,
            notice_only=True,
        )

    yield from check_levels(
        value=section.numevents,
        label='Events in DB',
        render_func=lambda v: format(v, ',d'),
        metric_name='checkpoint_eva_db_events_in_db',
    )


register.snmp_section(
    name='checkpoint_eva_db',
    parse_function=parse_checkpoint_eva_db,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.25.1',  # # CHECKPOINT-MIB::cpsemd
        oids=[
            '5',  # cpsemdCurrentDBSize
            '6',  # cpsemdDBCapacity
            '7',  # cpsemdNumEvents
            '8',  # cpsemdDBDiskSpace
            '10',  # cpsemdDBIsFull
            '11',  # disksize
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_eva_db',
    service_name='Eventia DB',
    discovery_function=discovery_checkpoint_eva_db,
    check_function=check_checkpoint_eva_db,
    check_default_parameters={
        'levels_db_size_percent': (80, 85),  # warn/crit if db size at 80/85 percent
    },
    check_ruleset_name='checkpoint_eva_db',
)
