#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2016-07-30
#
# monitor Check Point Eventia Analyzer job status
#
# 2018-03-12: code cleanup
# 2020-06-07: code cleanup
#             added parse function
# 2020-06-08: changed snmp-scan function
# 2021-08-14: rewritten for CMK 2.0
#
# sample string_table
# [
#  ['1', '{42DC9EE4-1529-4cb4-B4D9-E850AA328EDA}', 'All online jobs', 'running', '1', '192.168.10.89', 'Log',
#          '1', '3260', 'fw.log, Recent log time analyzed: 14Aug2021 15:27:35', '1545', 'eJobRunningAllOK', 'All OK.'],
#  ['2', '{42DC9EE4-1529-4cb4-B4D9-E850AA328EDA}', 'All online jobs', 'running', '1', '192.168.10.89', 'Audit',
#          '1', '204', 'fw.adtlog, Recent log time analyzed: 14Aug2021 15:45:58', '42', 'eJobRunningAllOK', 'All OK.']
# ]
#
# sample section
# {
#  '1': CheckpointEvaJob(jobid='{42DC9EE4-1529-4cb4-B4D9-E850AA328EDA}', jobname='All online jobs', jobstate='running',
#                        jobisonline=1, joblogserver='192.168.10.89', jobdatatype='Log', connectedtologserver=1,
#                        filename='fw.log', filecurrentposition=1545, statedescriptioncode='eJobRunningAllOK',
#                        statedescription='All OK.', recent_log_time=' Recent log time analyzed: 14Aug2021 15:27:35',
#                        numanalyzedlogs=3260),
#  '2': CheckpointEvaJob(jobid='{42DC9EE4-1529-4cb4-B4D9-E850AA328EDA}', jobname='All online jobs', jobstate='running',
#                        jobisonline=1, joblogserver='192.168.10.89', jobdatatype='Audit', connectedtologserver=1,
#                        filename='fw.adtlog', filecurrentposition=42, statedescriptioncode='eJobRunningAllOK',
#                        statedescription='All OK.', recent_log_time=' Recent log time analyzed: 14Aug2021 15:45:58',
#                        numanalyzedlogs=204)
# }
#
import time
from dataclasses import dataclass
from typing import Optional, Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    check_levels,
    State,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    get_rate,
    GetRateError,
    get_value_store,
    IgnoreResultsError,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointEvaJob:
    jobid: str
    jobname: str
    jobstate: str
    jobisonline: int
    joblogserver: str
    jobdatatype: str
    connectedtologserver: int
    filename: str
    filecurrentposition: int
    statedescriptioncode: str
    statedescription: str
    recent_log_time: str
    numanalyzedlogs: int


def parse_checkpoint_eva_jobs(string_table: StringTable) -> Dict[str, CheckpointEvaJob]:
    section = {}
    for job_entry in string_table:
        jobindex, jobid, jobname, jobstate, jobisonline, joblogserver, jobdatatype, connectedtologserver, \
            numanalyzedlogs, filename, filecurrentposition, statedescriptioncode, statedescription = job_entry

        if ',' in filename:
            filename, recent_log_time = filename.split(',')
        else:
            recent_log_time = 'unknown'

        section[jobindex] = CheckpointEvaJob(
            jobid=jobid,
            jobname=jobname,
            jobstate=jobstate,
            jobisonline=int(jobisonline),
            joblogserver=joblogserver,
            jobdatatype=jobdatatype,
            connectedtologserver=int(connectedtologserver),
            filename=filename,
            filecurrentposition=int(filecurrentposition),
            statedescriptioncode=statedescriptioncode,
            statedescription=statedescription,
            recent_log_time=recent_log_time,
            numanalyzedlogs=int(numanalyzedlogs)
        )
    return section


def discovery_checkpoint_eva_jobs(section: Dict[str, CheckpointEvaJob]) -> DiscoveryResult:
    for key in section.keys():
        yield Service(item=key)


def check_checkpoint_eva_jobs(item: str, section: Dict[str, CheckpointEvaJob]) -> Optional[CheckResult]:
    try:
        job_entry = section[item]
    except KeyError:
        return

    now_time = time.time()
    value_store = get_value_store()
    try:
        numanalyzedlogs_per_second = get_rate(
            value_store,
            f'checkpoint_eva_jobs_analyzed_logs{item}',
            now_time,
            job_entry.numanalyzedlogs, raise_overflow=False
        )
    except GetRateError:
        raise IgnoreResultsError('Initializing counters')

    yield Result(state=State.OK, notice=f'Job id: {job_entry.jobid}')
    yield Result(state=State.OK, notice=f'Job name: {job_entry.jobname}')
    if job_entry.jobisonline != 1:
        yield Result(state=State.CRIT, notice=f'Job State: {job_entry.jobstate}')
    else:
        yield Result(state=State.OK, notice=f'Job state: {job_entry.jobstate}')
    yield Result(state=State.OK, notice=f'Job type: {job_entry.jobdatatype}')
    yield Result(state=State.OK, notice=f'Job file: {job_entry.filename}')

    yield Result(state=State.OK, summary=f'Server: {job_entry.joblogserver}')
    yield from check_levels(
        value=numanalyzedlogs_per_second,
        label='Analyzed logs',
        render_func=lambda v: f'{v:0.2f}/s',
        metric_name='checkpoint_eva_jobs_analyzed_log'
    )
    yield from check_levels(
        value=job_entry.filecurrentposition,
        label='Position in file',
        render_func=lambda v: f'{v:.0f}',
        metric_name='checkpoint_eva_jobs_position_in_file'
    )

    yield Result(state=State.OK, notice=f'Recent log time: {job_entry.recent_log_time}')
    yield Result(state=State.OK, notice=f'State description code: {job_entry.statedescriptioncode}')
    yield Result(state=State.OK, notice=f'State description: {job_entry.statedescription}')

    if job_entry.connectedtologserver != 1:
        yield Result(state=State.CRIT, notice='Not connected to log server')


register.snmp_section(
    name='checkpoint_eva_jobs',
    parse_function=parse_checkpoint_eva_jobs,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.25.2.4.1',   # CHECKPOINT-MIB::cpseadJobsEntry
        oids=[
            '1',  # cpseadJobIndex
            '2',  # cpseadJobID
            '3',  # cpseadJobName
            '4',  # cpseadJobState
            '5',  # cpseadJobIsOnline
            '6',  # cpseadJobLogServer
            '7',  # cpseadJobDataType
            '8',  # cpseadConnectedToLogServer
            '9',  # cpseadNumAnalyzedLogs
            '10',  # cpseadFileName
            '11',  # cpseadFileCurrentPosition
            '12',  # cpseadStateDescriptionCode
            '13',  # cpseadStateDescription
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_eva_jobs',
    service_name='Eventia job %s',
    discovery_function=discovery_checkpoint_eva_jobs,
    check_function=check_checkpoint_eva_jobs,
)
